import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';
import 'package:modular_system_core/src/widget/entity_table/table_extra_options.dart';
import 'package:modular_system_core/src/widget/entity_table/table_tile.dart';
export 'viewmodel.dart';
export './app_bar.dart';

class EntityTable<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTable({
    required this.appViewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    this.filtersSheetBuilder,
    this.selectorActionsBuilder,
    this.editorBuilder,
    this.extraOptions,
  });

  // properties
  final AppViewModel appViewModel;
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Duration transitionDuration;
  final Widget? extraOptions;

  // builders
  final TableItemBuilder<T> itemBuilder;
  final TableWidgetBuilder<T> formBuilder;

  // optional builders
  final AppBottomSheet Function(
    EntityTableViewModel<T> viewModel,
  )? filtersSheetBuilder;

  final List<SingleEntryMap<Widget, Function>> Function(
      EntityTableViewModel<T> viewModel)? selectorActionsBuilder;

  final TableEditorBuilder<T>? editorBuilder;

  @override
  State<EntityTable> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableState<T>(
      appViewModel: appViewModel,
      viewModel: getViewModel(),
    );
  }

  EntityTableViewModel<T> getViewModel() {
    return EntityTableViewModel(
      properties: TableVmProperties(
        update: update,
        itemBuilder: itemBuilder,
        formBuilder: formBuilder,
        transitionDuration: transitionDuration,
        filtersSheetBuilder:
            filtersSheetBuilder ?? (vm) => AppBottomSheet.empityRounded(),
        selectorActionsBuilder: selectorActionsBuilder ?? (vm) => [],
        editorBuilder: editorBuilder,
        extraOptions: extraOptions,
      ),
      config: TableVmConfig(
        hasExtraOptions: extraOptions != null,
      ),
    );
  }
}

class _EntityTableState<T> extends State<EntityTable> {
  _EntityTableState({
    required this.appViewModel,
    required this.viewModel,
  }) {
    subscribeToEvents();
    setUpdateFuture();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  bool selectorOn = false;
  late Widget body = getTableBody();
  late Future updateTableFuture;
  late List<Widget> listData = [];

  void subscribeToEvents() {
    viewModel.events.updateEvent.subscribe(onUpdate);
    viewModel.events.updateStateChanged.subscribe(onUpdateStateChanged);
    viewModel.events.showTable.subscribe(onShowTable);
    viewModel.events.showForm.subscribe(onShowForm);
    viewModel.events.showEditor.subscribe(onShowEditor);
  }

  void unsubscribeToEvents() {
    viewModel.events.updateEvent.unsubscribe(onUpdate);
    viewModel.events.updateStateChanged.unsubscribe(onUpdateStateChanged);
    viewModel.events.showTable.unsubscribe(onShowTable);
    viewModel.events.showForm.unsubscribe(onShowForm);
    viewModel.events.showEditor.unsubscribe(onShowEditor);
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedSwitcher(
      duration: viewModel.properties.transitionDuration,
      child: body,
    );
  }

  Widget getTableBody() {
    return Container(
      color: appViewModel.colorScheme.background,
      child: CustomScrollView(
        slivers: [
          EntityTableAppBar(
            viewModel: viewModel,
            appViewModel: appViewModel,
            updateTableFuture: updateTableFuture,
          ),
          FutureBuilder(
            future: updateTableFuture,
            builder: (context, snapshot) {
              if (snapshot.connectionState == ConnectionState.done) {
                if (snapshot.hasError) {
                  return SliverToBoxAdapter(
                    child: Text(
                      "ERRO",
                      style: appViewModel.errorTextStyle(AppTextType.subtitle),
                    ),
                  );
                }
                return SliverList(
                  key: UniqueKey(),
                  delegate: SliverChildListDelegate(listData),
                );
              }
              return SliverToBoxAdapter(
                child: appViewModel.loading(),
              );
            },
          ),
        ],
      ),
    );
  }

  void setBody(Widget widget) {
    setState(() {
      body = widget;
    });
  }

  void setListData() {
    listData.clear();
    listData.add(TableExtraOptions(
      appViewModel: appViewModel,
      viewModel: viewModel,
      child: viewModel.properties.extraOptions,
    ));
    for (var index = 0; index < viewModel.state.data.length; index++) {
      listData.add(TableTile(
        index: index,
        appViewModel: appViewModel,
        viewModel: viewModel,
        child: viewModel.properties.itemBuilder(
          context,
          viewModel.state.data[index],
          viewModel,
        ),
      ));
    }
  }

  // Event Handlers
  void onShowTable(dynamic data) => setBody(getTableBody());

  void onShowForm(dynamic data) {
    setBody(viewModel.properties.formBuilder(
      context,
      viewModel,
    ));
  }

  void onShowEditor(T? data) {
    if (viewModel.properties.editorBuilder != null) {
      setBody(viewModel.properties.editorBuilder!(
        context,
        data!,
        viewModel,
      ));
    }
  }

  // Event Handlers
  void onUpdate(dynamic data) {
    setState(() {
      viewModel.state.selectedTiles.clear();
      setUpdateFuture();
      viewModel.events.updateStateChanged.invoke();
    });
  }

  void onUpdateStateChanged(dynamic data) {
    onShowTable(data);
  }

  void setUpdateFuture() {
    updateTableFuture = Future(() async {
      try {
        var map = await viewModel.properties.update(viewModel.state.search);
        viewModel.state.paginationOut = map.key;
        viewModel.state.data = map.value;
        setListData();
      } catch (e) {
        appViewModel.appEvents.errorEvent.invoke(e);
      }
    });
    viewModel.state.updateFuture = Future(() async {
      try {
        var map = await viewModel.properties.update(viewModel.state.search);
        viewModel.state.paginationOut = map.key;
        viewModel.state.data = map.value;
        setListData();
      } catch (e) {
        appViewModel.appEvents.errorEvent.invoke(e);
      }
    });
  }
}
